import java.util.*;
import java.io.*;
import java.util.Date;
import java.security.MessageDigest;

// borrowed some code from;
// - http://www.java-forums.org/java-io/7169-how-change-date-modification-file.html
// - http://www.rgagnon.com/javadetails/java-0416.html

public class FileModChanger
{
   	static File startingDirectory;
  	static File compareDirectory;
  	
  	static long totalProcessedFiles = 0;
  	static long totalChangedFiles = 0;
  	static int currentFileProgress = 0;
  	static int scanningProgress = 0;
  	static int scanningTotalProgress = 0;
  	static int totalFilesToProcess;
  	static boolean dryrun = true;

	public static void main(String[] args) throws FileNotFoundException
	{

		if(args.length<2||(args.length==3&&!args[2].equals("real")))
		{
			System.out.println("\nUSAGE (dry run, NO file changes made):\n  FileModChanger <source folder with  original dates> <target folder with new/erased dates>");
			System.out.println("\nUSAGE (real run,file date changes WILL be made):\n  FileModChanger <source folder with  original dates> <target folder with new/erased dates> real\n\n");
			System.exit(0);
		}
		
		if(args.length==3&&null!=args[2]&&args[2].equals("real"))
		{
			dryrun=false;
			System.out.println("Starting REAL run....");
			System.out.println("Starting REAL run....");
			System.out.println("Starting REAL run....");
		}
		else
		{
			System.out.println("Starting DRY run....");
			System.out.println("Starting DRY run....");
			System.out.println("Starting DRY run....");
		}
		

   		startingDirectory = new File(args[0]);
  		compareDirectory  = new File(args[1]);
    	
    	System.out.println("Starting to retrieve list of files (this may take a LONG time)...");
    	
    	List<File> files = getFileListing(startingDirectory);
    	
    	totalFilesToProcess = files.size();

		System.out.println("\nDONE creating the list of files, starting comparison...\n");

    	//print out all file names, in the the order of File.compareTo()
    	for(File file : files )
    	{
      		//System.out.println(file);
      		compareFileDates(file);
    	}
    	
    	System.out.println("DONE processing all files..");
    }

  	static public List<File> getFileListing(File aStartingDir) throws FileNotFoundException 
  	{
    	validateDirectory(aStartingDir);
    	List<File> result = getFileListingNoSort(aStartingDir);
    	Collections.sort(result);
    	
    	return result;
  	}

  	// PRIVATE //
  	static private List<File> getFileListingNoSort(File aStartingDir) throws FileNotFoundException 
  	{
    	List<File> result = new ArrayList<File>();
    	File[] filesAndDirs = aStartingDir.listFiles();
    	List<File> filesDirs = Arrays.asList(filesAndDirs);
    	
    	for(File file : filesDirs) 
    	{
      		result.add(file); //always add, even if directory
      		if ( ! file.isFile() ) 
      		{
        		//must be a directory
        		//recursive call!
        		List<File> deeperList = getFileListingNoSort(file);
        		result.addAll(deeperList);
        		
        		//print progress as it could take very long
        		System.out.print("\r");

        		System.out.print(scanningTotalProgress + " folders processed ");

        		for(int i=0;i<scanningProgress;i++)
        		{
        			System.out.print(".");
        		}

        		scanningProgress++;
        		scanningTotalProgress++;
        		if(scanningProgress>10)
        		{
        			scanningProgress=0;
        		}
      		}
    	}
    
    	return result;
    }

  	/**
  	* Directory is valid if it exists, does not represent a file, and can be read.
  	*/
  	static private void validateDirectory (File aDirectory) throws FileNotFoundException 
  	{
    	if (aDirectory == null) 
    	{
      		throw new IllegalArgumentException("Directory should not be null.");
    	}
    	if (!aDirectory.exists()) 
    	{
      		throw new FileNotFoundException("Directory does not exist: " + aDirectory);
    	}
    	if (!aDirectory.isDirectory()) 
    	{
      		throw new IllegalArgumentException("Is not a directory: " + aDirectory);
    	}
    	if (!aDirectory.canRead()) 
    	{
      		throw new IllegalArgumentException("Directory cannot be read: " + aDirectory);
    	}
  	}
  	
   	private static byte[] createChecksum(String filename) throws Exception
   	{
     	File toCheck = new File(filename);
     	if(toCheck.isDirectory())
     	{
     		//its a directory so we calculate md5 on name not contents
     		return filename.getBytes();
     	}
     	else
     	{
     		InputStream fis =  new FileInputStream(filename);

     		byte[] buffer = new byte[1024];
     		MessageDigest complete = MessageDigest.getInstance("MD5");
     		int numRead;
     		do {
      			numRead = fis.read(buffer);
      			if (numRead > 0) {
        			complete.update(buffer, 0, numRead);
        		}
      		} while (numRead != -1);
     		fis.close();
     
     		return complete.digest();
   		}
   	}

   	// there is a faster way (and more cumbersome) to convert
   	// a byte array to a HEX string
   	public static String getMD5Checksum(String filename) throws Exception 
   	{
     	byte[] b = createChecksum(filename);
     	String result = "";
     	for (int i=0; i < b.length; i++) {
       		result +=
          	Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 );
      	}
     
     	return result;
   	}
   	
   	private static boolean fileInBothSources(File sourceFile)
   	{   		
   		File checkFile = new File(compareDirectory + "/" + sourceFile.toString().replace(startingDirectory.toString(),""));
   		return checkFile.exists();
   	}

   	private static boolean fileTheSameInBothSources(File sourceFile)
   	{   		
   		File checkFile = new File(compareDirectory + "/" + sourceFile.toString().replace(startingDirectory.toString(),""));   		
   		
   		try{
   			//System.out.println(getMD5Checksum(sourceFile.toString()) + " vs " + getMD5Checksum(checkFile.toString()));
   			return getMD5Checksum(sourceFile.toString()).equals(getMD5Checksum(checkFile.toString()));
   		}
   		catch(Exception e)
   		{
				System.out.println("weird....;\n\n" + e.toString());
				e.printStackTrace();   			
   		}
   		return false; //crap!
   	}

   	private static boolean fileDateTheSameInBothSources(File sourceFile)
   	{   		
   		File checkFile = new File(compareDirectory + "/" + sourceFile.toString().replace(startingDirectory.toString(),""));   		
   		
   		//if(checkFile.lastModified()!=sourceFile.lastModified())
   		//{
   		//	System.out.println(new Date(checkFile.lastModified()) + " is different to " + new Date(sourceFile.lastModified()));
   		//}

   		return checkFile.lastModified()==sourceFile.lastModified();

   	}

   	private static boolean showFileDetails(File sourceFile)
   	{   		
   		File checkFile = new File(compareDirectory + "/" + sourceFile.toString().replace(startingDirectory.toString(),""));   
   		
   		String sourceFileNameDisplay = sourceFile.toString();
   		String targetFileNameDisplay = checkFile.toString();
   		
   		if(targetFileNameDisplay.length()>64){
   			targetFileNameDisplay = targetFileNameDisplay.substring(1,32) + "[...]" + targetFileNameDisplay.substring(targetFileNameDisplay.length()-33);
   		}
   		if(sourceFileNameDisplay.length()>64){
   			sourceFileNameDisplay = sourceFileNameDisplay.substring(1,32) + "[...]" + sourceFileNameDisplay.substring(sourceFileNameDisplay.length()-33);
   		}
   		
  		System.out.println("\n\nAbout to operate on " + sourceFileNameDisplay + ":");

   		   		
		try
		{
		System.out.println(" SRC:" + new Date(sourceFile.lastModified()) + " - " + getMD5Checksum(sourceFile.toString()) + " - " + sourceFileNameDisplay + "\n TRG:" + new Date(checkFile.lastModified()) + " - " + getMD5Checksum(checkFile.toString()) + " - " + targetFileNameDisplay);
		}
		catch(Exception e)
		{
		}

   		return checkFile.lastModified()==sourceFile.lastModified();

   	}


   	private static void alignFileDates(File sourceFile)
   	{   		
   		File targetFile = new File(compareDirectory + "/" + sourceFile.toString().replace(startingDirectory.toString(),""));   
   		   		
		try
		{
			long sourceModifiedDate = sourceFile.lastModified();
			System.out.println(" RESULT:\n   - BEFORE:    " + new Date(targetFile.lastModified()));

			targetFile.setLastModified(sourceModifiedDate);
			
			targetFile = new File(compareDirectory + "/" + sourceFile.toString().replace(startingDirectory.toString(),""));
			
			System.out.println("   - AFTER:     " + new Date(targetFile.lastModified()));		
		}
		catch(Exception e)
		{
				System.out.println("SEVERE;\n\n" + e.toString());
				e.printStackTrace();			
		}

   	}

    // lets do it.
   	static private void compareFileDates(File theFile) 
  	{
  			try
  			{
  				//long modifiedTime = theFile.lastModified();
  				
  				// does the file exist in both sources, is it the same and is the date DIFFERENT?
  				if(fileInBothSources(theFile)&&fileTheSameInBothSources(theFile)&&!fileDateTheSameInBothSources(theFile))
  				{
  					showFileDetails(theFile);
  					
  					if(!dryrun)
  					{
  						alignFileDates(theFile);
  					}
  					totalChangedFiles++;  					
  				}
  				
				//System.out.println(new Date(modifiedTime) + " - " + getMD5Checksum(theFile.toString()) + " - " + theFile );
			}
			catch (Exception e)
			{
				System.out.println("crap;\n\n" + e.toString());
				e.printStackTrace();
			}

  			totalProcessedFiles++;
  			currentFileProgress++;
  			
  			if(currentFileProgress>99)
  			{
  				System.out.println("\nCHANGED 100 FILES. (" + totalProcessedFiles + " total files processed, " + (totalFilesToProcess - totalProcessedFiles) + " files left to go , " + totalChangedFiles + " total files changed)");
  				currentFileProgress=0;
  			}
  	}
} 

